@@ -3,7 +3,7 @@
 See http://www-huber.embl.de/users/anders/HTSeq for documentation.
 """
 
-import itertools, warnings, os
+import itertools, warnings, os, shlex
 
 try:
    from _HTSeq import *
@@ -77,19 +77,6 @@ class FileOrSequence( object ):
       else:
          return "line %d" % self.line_no
          
-class GeneratedSequence( object ):
-   """GeneratedSequence is a little helper class to get "respawnable"
-   iterator generators. Pass its constructor a function that generates
-   an iterator, and its arguments, and it will call this functionion 
-   whenever an iterator is requested.
-   """
-   
-   def __init__( self, generator, *args ):
-      self.generator = generator
-      self.args = args
-      
-   def __iter__( self ):
-      return self.generator( *self.args )
 
 #########################
 ## Features
@@ -578,6 +565,8 @@ class GenomicArrayOfSets( GenomicArray ):
                 
 def pair_SAM_alignments( alignments, bundle=False ):
 
+   mate_missing_count = [0]
+
    def process_list( almnt_list ):
       while len( almnt_list ) > 0:
          a1 = almnt_list.pop( 0 )
@@ -594,8 +583,10 @@ def pair_SAM_alignments( alignments, bundle=False ):
                break
          else:
             if a1.mate_aligned:
-               warnings.warn( "Read " + a1.read.name + " claims to have an aligned mate " +
-                  "which could not be found. (Is the SAM file properly sorted?)" )
+               mate_missing_count[0] += 1
+               if mate_missing_count[0] == 1:
+                  warnings.warn( "Read " + a1.read.name + " claims to have an aligned mate " +
+                     "which could not be found in an adjacent line." )
             a2 = None
          if a2 is not None:
             almnt_list.remove( a2 )
@@ -627,6 +618,71 @@ def pair_SAM_alignments( alignments, bundle=False ):
    else:
       for p in process_list( almnt_list ):
          yield p
+   if mate_missing_count[0] > 1:
+      warnings.warn( "%d reads with missing mate encountered." % mate_missing_count[0] )
+
+
+def pair_SAM_alignments_with_buffer( alignments, max_buffer_size=3000000 ):
+
+   almnt_buffer = {}
+   ambiguous_pairing_counter = 0
+   for almnt in alignments:
+
+      if not almnt.paired_end:
+         raise ValueError, "Sequence of paired-end alignments expected, but got single-end alignment."
+      if almnt.pe_which == "unknown":
+         raise ValueError, "Cannot process paired-end alignment found with 'unknown' 'pe_which' status."
+
+      matekey = ( 
+         almnt.read.name, 
+         "second" if almnt.pe_which == "first" else "first",
+         almnt.mate_start.chrom if almnt.mate_aligned else None, 
+         almnt.mate_start.pos if almnt.mate_aligned else None, 
+         almnt.iv.chrom if almnt.aligned else None, 
+         almnt.iv.start if almnt.aligned else None, 
+         -almnt.inferred_insert_size if almnt.aligned and almnt.mate_aligned else None )
+
+      if matekey in almnt_buffer:
+         if len( almnt_buffer[ matekey ] ) == 1:            
+            mate = almnt_buffer[ matekey ][ 0 ]
+            del almnt_buffer[ matekey ]
+         else:
+            mate = almnt_buffer[ matekey ].pop( 0 )
+            if ambiguous_pairing_counter == 0:
+               ambiguous_pairing_first_occurance = matekey
+            ambiguous_pairing_counter += 1
+         if almnt.pe_which == "first":
+            yield ( almnt, mate )
+         else:
+            yield ( mate, almnt )
+      else:
+         almntkey = ( 
+            almnt.read.name, almnt.pe_which, 
+            almnt.iv.chrom if almnt.aligned else None, 
+            almnt.iv.start if almnt.aligned else None, 
+            almnt.mate_start.chrom if almnt.mate_aligned else None, 
+            almnt.mate_start.pos if almnt.mate_aligned else None, 
+            almnt.inferred_insert_size if almnt.aligned and almnt.mate_aligned else None )
+         if almntkey not in almnt_buffer:
+            almnt_buffer[ almntkey ] = [ almnt ]
+         else:
+            almnt_buffer[ almntkey ].append( almnt )
+         if len(almnt_buffer) > max_buffer_size:
+            raise ValueError, "Maximum alignment buffer size exceeded while pairing SAM alignments."
+
+   if len(almnt_buffer) > 0:
+      warnings.warn( "Mate records missing for %d records; first such record: %s." % 
+         ( len(almnt_buffer), str( almnt_buffer.values()[0][0] ) ) )
+      for almnt_list in almnt_buffer.values():
+         for almnt in almnt_list:
+            if almnt.pe_which == "first":
+               yield ( almnt, None )
+            else:
+               yield ( None, almnt )
+
+   if ambiguous_pairing_counter > 0:
+      warnings.warn( "Mate pairing was ambiguous for %d records; mate key for first such record: %s." %
+         ( ambiguous_pairing_counter, str( ambiguous_pairing_first_occurance ) ) )
 
 
 ###########################
@@ -819,6 +875,57 @@ class VCF_Reader( FileOrSequence ):
             vc = VariantCall.fromline( line, self.nsamples, self.sampleids )
             yield vc
 
+
+class WIG_Reader( FileOrSequence ):
+
+    def __init__( self, filename_or_sequence, verbose = True ):
+        FileOrSequence.__init__( self, filename_or_sequence )
+        self.attributes = {}       
+        self.stepType = 'none'
+        self.verbose = verbose
+        
+    def __iter__( self ):
+        span = 1
+        pos = None
+        step = None
+        chrom = None
+        for line in FileOrSequence.__iter__( self ):
+            if line.startswith( 'track' ):
+                fields = shlex.split(line)[1:]
+                self.attributes = dict([(p[0], p[1].strip('"')) for p in [x.split("=") for x in fields]])
+            elif line.startswith( 'fixedStep' ): # do fixed step stuff
+                self.stepType = 'fixed'
+                fields = shlex.split(line)[1:]
+                declarations = dict([(p[0], p[1].strip('"')) for p in [x.split("=") for x in fields]])
+                pos = int(declarations['start'])
+                step = int(declarations['step'])
+                chrom = declarations['chrom']
+                if 'span' in declarations:
+                    span = int(declarations['span'])
+                else:
+                    span = 1
+            elif line.startswith( 'variableStep' ): # do variable step stuff
+                self.stepType = 'variable'
+                fields = shlex.split(line)[1:]
+                declarations = dict([(p[0], p[1].strip('"')) for p in [x.split("=") for x in fields]])
+                chrom = declarations['chrom']
+                if 'span' in declarations:
+                    span = int(declarations['span'])
+                else:
+                    span = 1
+            elif line.startswith( 'browser' ) or line.startswith( '#' ): #Comment or ignored
+                if self.verbose:
+                    print "Ignored line:", line
+                continue
+            else:
+                if self.stepType == 'fixed':
+                    yield ( GenomicInterval( chrom, pos, pos + span, '.' ), float(line.strip()) )
+                    pos += step
+                elif self.stepType == 'variable':
+                    tmp = line.strip().split(" ")
+                    pos = int(tmp[0])
+                    yield ( GenomicInterval( chrom, pos, pos + span, '.' ), float(tmp[1]) )
+            
 class BAM_Reader( object ):
 
     def __init__( self, filename ):
